
import sys
import os
sys.path.append(os.path.dirname(__file__))
import numpy as np
import csv
from datetime import datetime as dt
from util import (get_root_path,get_single_dict,single_keys_sorted)

from pyftg.struct.enumerate import *
from pyftg.ai_interface import AIInterface
from pyftg.struct import *
from pyftg.struct.attack_data import *
from io import TextIOWrapper
import ntpath
from typing import List,Dict,Tuple
from copy import copy

def get_time_str():
    tdatetime = dt.now()
    tstr=tdatetime.strftime('%m%d_%H%M')
    return tstr
def get_file(prefix:str="")->TextIOWrapper:
    root_path=get_root_path("DareFightingICE",__file__)
    log_path=f"{root_path}mylog{ntpath.sep}"
    os.makedirs(log_path,exist_ok=True)
    return open(f"{log_path}data{prefix}_{get_time_str()}.csv","w",newline='\n')

class MyKey:
    def __init__(self) -> None:
        self.keys={}
        pass
    def get(self,s:str)->Key:
        if s in self.keys:
            return self.keys[s]
        k:Key=Key()
        for c in s:
            if c=='a':
                k.A=True
            elif c=='b':
                k.B=True
            elif c=='c':
                k.C=True
            elif c=='U':
                k.U=True
            elif c=='D':
                k.D=True
            elif c=='F':
                k.R=True
            elif c=='B':
                k.L=True
        self.keys[s]=k
        return k
mykey=MyKey()

class PyThunder(AIInterface):
    is_survey=False
    memory_size=10
    # wait_count=100
    # dash_count=30
    # dash_loop_count=1
    def __init__(self):
        print("PyThunder")
        self.buffers=[]
        super().__init__()
        self.blind_flag = True
        
        self.keys=[
            #
            mykey.get("D"),mykey.get("DB"),mykey.get("Bb"),mykey.get("N"),
            #
            mykey.get("Db"),mykey.get("N"),
            #
            mykey.get("D"),mykey.get("DF"),mykey.get("Fb"),mykey.get("N"),
            #
            mykey.get("Db"),mykey.get("N"),
        ]
        self.index:int=0
        self.index_reset_count:int=0

        self.non_delay_frame_data: FrameData=None
        self.key_id_dict=dict()
        self.volumes=[[],[]]

    def is_blind(self) -> bool:
        return self.blind_flag

    def initialize(self, game_data: GameData, player_number: bool):
        self.cc = CommandCenter()
        self.key = Key()
        self.player = player_number
        self.game_data=game_data



    def input(self) -> Key:
        return self.key

    def get_information(self, frame_data: FrameData, is_control: bool, non_delay_frame_data: FrameData):
        self.non_delay_frame_data=non_delay_frame_data
        self.frame_data = frame_data
        self.cc.set_frame_data(self.frame_data, self.player)

    def get_screen_data(self, screen_data: ScreenData):
        self.screen_data = screen_data

    def get_audio_data(self, audio_data: AudioData):
        # process audio
        try:
            byte_data = audio_data.raw_data_as_bytes
            np_array = np.frombuffer(byte_data, dtype=np.float32)
            np_array = np_array.reshape((2, 1024))
        except Exception as ex:
            np_array = np.zeros((2,1024))
        np_array=np.abs(np_array)
        volume=[np.sum(per_ar) for per_ar in np_array]
        self.volumes.append(volume)
        if len(self.volumes)>PyThunder.memory_size:
            self.volumes=self.volumes[1:]

    def processing(self):
        if self.non_delay_frame_data is not None:
            PyThunder.is_survey=True # non_delay_frame_dataが与えられるのは自分の環境でデータ集めするときだけa
            if not self.non_delay_frame_data.empty_flag:
                frame_data_for_log=copy(self.non_delay_frame_data)
                if not self.player:
                    print("reveresed")
                    frame_data_for_log.character_data=frame_data_for_log.character_data[::-1]
                
                audio_dict={}
                if len(self.volumes)==PyThunder.memory_size:
                    ta=np.array(self.volumes)
                    ta=ta.sum(axis=1)       
                    audio_dict["volumes"]=ta          
                    audio_dict["gap"]=ta[0]-ta[1]
                    audio_dict["sum"]=np.abs(ta[0])+np.abs(ta[1])

                org_data={"audio":audio_dict,"frame":frame_data_for_log}
                single_dict=get_single_dict(org_data)
                self.buffers.append(single_dict)
                        
        self.key=self.keys[self.index]
        self.index+=1
        if self.index==len(self.keys) :
            self.index=0
            self.index_reset_count+=1
            # if self.index_reset_count>=PyThunder.dash_loop_count:
            #     self.keys=self.normal_keys
    
    def round_end(self, round_result: RoundResult):
        print(round_result.remaining_hps[0])
        print(round_result.remaining_hps[1])
        print(round_result.elapsed_frame)
        if PyThunder.is_survey:
            with get_file(f"_{self.game_data.ai_names[0]}_{self.game_data.ai_names[1]}") as f:
                writer=csv.writer(f)
                name_set=set()
                for frame_dict in self.buffers:
                    for key,value in frame_dict.items():
                        name_set.add(key)
                sorted_names:list=single_keys_sorted(name_set)
                writer.writerow(sorted_names)
                for frame_dict in self.buffers:
                    buffer=[frame_dict[name] if name in frame_dict.keys() else None for name in sorted_names]
                    writer.writerow(buffer)

    def game_end(self):
        pass



